library(phylobase)
library(ggplot2)
library(phangorn)
library(distory)
library(reshape2)


# Function to load all final bipartition tree files from directory
loadtrees <- function(dataset)
{
  trees.list <- list()
  trees.list[[1]] <- read.tree( paste(basepath, dataset, "/RAXML/brlen_reanalysis/RAxML_bipartitions.", pfschemes[1], ".", fsuffix, sep="") )  # PFbic
  trees.list[[2]] <- read.tree( paste(basepath, dataset, "/RAXML/brlen_reanalysis/RAxML_bipartitions.", pfschemes[2], ".", fsuffix, sep="") )  # PFaic
  trees.list[[3]] <- read.tree( paste(basepath, dataset, "/RAXML/brlen_reanalysis/RAxML_bipartitions.", pfschemes[3], ".", fsuffix, sep="") )  # PFall
  trees.list[[4]] <- read.tree( paste(basepath, dataset, "/RAXML/brlen_reanalysis/RAxML_bipartitions.", pfschemes[4], ".", fsuffix, sep="") )  # PFnone
  cat( paste(dataset, "found trees\n") )
  return(trees.list)
}

loadtreesRooted <- function(dataset)
{
  trees = read.tree( paste(basepath, dataset, "/RAXML/brlen_reanalysis/best_trees_reanalysis_rooted.txt", sep="")  )
  cat( paste(dataset, "found rooted trees\n") )
  return(trees)
}

# executes TreeCmp java program. Should take only seconds per dataset.
# -- output is saved in a tab-delimited txt file which is then opened and returned for parsing
topo_distances <- function(dataset)
{
  cmd   <- paste( "java -jar ", treeCmppath, " -m -d ms rf pd qt -i ", basepath, dataset, "/RAXML/brlen_reanalysis/best_trees_reanalysis3.txt -o ", basepath, dataset, "/RAXML/brlen_reanalysis/treedistances3.txt -N", sep="")
  system(cmd, wait=TRUE) 
  distances <- read.table( paste(basepath, dataset, "/RAXML/brlen_reanalysis/treedistances3.txt", sep=""), header=TRUE)
  return(distances)
}


# Rob Lanfear's function to calculate the slope of the regression of branch lengths on branch depth for a single tree.
# - must use ROOTED trees for this analysis.
node.depth.slope <- function(tree){
  
  n = node.depth(tree, method = 2)
  n = n[-c(which(n == max(n)))] # remove the root node, which doesn't have an edge associated
  d = data.frame(bl = tree$edge.length, depth = n)
  slope = lm(d$depth ~ log(d$bl))$coefficients[[2]]
  return(slope)		
}

# opens the best model (GTR+G or GTR+I+G) PartitionFinder output file for each of the 4 schemes per dataset
# and grabs the Number of params and Number of subsets fields.
library("stringr", lib.loc="~/R/win-library/3.0")
data_info <- function(dataset, pfscheme_num)
{
  model   <- ""
  if(bestmodels[[dataset]][pfscheme_num] == 1)
  {
    model <- "GTRIG"
  }
  else
  {
    model <- "GTRG"
  }
  path        <- paste(basepath,dataset,'/',pfschemes[pfscheme_num],'_',model,'.txt', sep = "")
  subsetsline <- grep( 'subsets', readLines(path), value=T )
  subsets     <- as.numeric(str_extract(subsetsline,pattern = "[0-9]{1,}"))
  paramsline  <- grep( 'params', readLines(path), value=T )
  params      <- as.numeric(str_extract(paramsline,pattern = "[0-9]{1,}"))
  return( c(params, subsets) )
}

doOneDataset <- function(d, vs_tree)
{
  print(vs_tree)
  trees         <- loadtrees( d )
  trees.rooted  <- loadtreesRooted( d )
  trees.phylo4  <- lapply(trees, as, "phylo4")  #phylo4 version of the trees. Useful for data mining
  names(trees)  <- pfschemes
  names(trees.rooted)  <- pfschemes
  
  
  distances     <- topo_distances( d )
  
  vs            <- which(pfschemes == vs_tree)
  
  outdf <- data.frame( dataset=character(), PF.scheme=character(),  dist.MS=numeric(), dist.RF=numeric(), dist.PD=numeric(), dist.KF=numeric(),
                       nodes.total=numeric(), nodes.internal=numeric(), nodes.numdiff=numeric(), Diffs.nodes=character(), 
                       Diffs.BS=character(), Diffs.BS.mean=numeric(), Diffs.BL=character(), Diffs.BL.mean=numeric(), 
                       Sames.BS=character(), Sames.BS.mean=numeric(), Sames.BL=character(), Sames.BL.mean=numeric(), 
                       BL.mean=numeric(), BS.mean=numeric(), BL.all=character(), BS.all=character(), params=numeric(), subsets=numeric(), 
                       slope.BLdepth = numeric(),
                       row.names=NULL, stringsAsFactors=FALSE )

  
  numtips <- length(trees[[vs_tree]]$tip.label)
  
  for(i in 1:length(pfschemes)) # (1:"PFbic" (vs), 2:"PFaicc", 3:"PFall", 4:"PFnone")
  {
    
    # distances vs BIC tree (Matching Split, Robinson Foulds, Path Distance, Khuner Felsenstein)
    
    if(i == vs)  #e.g. comparing BIC to BIC
    {
      MS <- 0
      RF <- 0
      PD <- 0
      KF <- dist.topo(trees[[i]], trees[[vs_tree]])
    }
    else
    {   
      dist <- subset(distances, (Tree1==i | Tree2==i) & (Tree1==vs | Tree2==vs), c(2,3,5,8,11))
      MS <- dist[1,3]
      RF <- dist[1,4]
      PD <- dist[1,5]
      KF <- dist.topo(trees[[i]], trees[[vs_tree]]) # Kuhner & Felsenstein branch score as per reviewer suggestion
    }
    

    
    # summary stats
    #Binternal <- which( trees[[i]]$edge[,2] > length(trees[[i]]$tip.label) )  #nodes are labeled by external nodes first, from 1 to num tips, and then internal nodes.
    #Bexternal <- which( trees[[i]]$edge[,2] <= length(trees[[i]]$tip.label) )
    BL_all      <- trees[[i]]$edge.length
    #BL_int      <- trees[[i]]$edge.length[Binternal]
    #BL_ext      <- trees[[i]]$edge.length[Bexternal]
    BL_mean     <- mean( BL_all )
    #BL_int_mean <- mean( BL_int )
    #BL_ext_mean <- mean( BL_ext )
    nnodes      <- trees[[i]]$Nnode
    totnodes    <- trees[[i]]$Nnode + length(trees[[i]]$tip)
    BS_all      <- as.numeric(trees[[i]]$node.label[-1])
    BS_mean     <- mean( BS_all )
    
    # treedist[[1]] = Robinson Foulds distance. I am modifying this by dividing by 2 and then by ntaxa
#     RFdist_aicc   <- ( treedist(trees[[i]], trees[["PFaicc"]])[[1]] ) / (2*(numtips-3))  
#     RFdist_all    <- ( treedist(trees[[i]], trees[["PFall"]])[[1]] ) / (2*(numtips-3))  
#     RFdist_bic    <- ( treedist(trees[[i]], trees[["PFbic"]])[[1]] ) / (2*(numtips-3))   
#     RFdist_none   <- ( treedist(trees[[i]], trees[["PFnone"]])[[1]] ) / (2*(numtips-3))  
#     # treedist[[4]] = quadratic (weighted) path distance
#     WPD_aicc  <- treedist(trees[[i]], trees[["PFaicc"]])[[3]]   
#     WPD_all   <- treedist(trees[[i]], trees[["PFall"]])[[3]]
#     WPD_bic   <- treedist(trees[[i]], trees[["PFbic"]])[[3]]
#     WPD_none  <- treedist(trees[[i]], trees[["PFnone"]])[[3]]
    
    # COMPARE THIS TREE TO VERSUS tree
    # step 1 - get the list of internal nodes that exist in this tree but NOT in the Versus tree          
    # step 2 - get the internal node numbers for the diffs nodes. This is diffs[] + length(tip.label)
    # step 3 - look in tree$edge matrix for values in the 2nd column that match the diff_nodes. The matching row number is the edge number
    # step 4 - get the edge length and bootstrap vals for the edge IDs in diffs_edges.
    
    diffs           <- prop.clades( trees[[i]],trees[[vs_tree]], part=NULL, rooted=FALSE )
    diffs_nodes     <- which(diffs==0) + numtips    
    diffs_edges     <- which(trees[[i]]$edge[,2] %in% diffs_nodes)    
    diffs_BL        <- trees[[i]]$edge.length[diffs_edges]
    diffs_BS        <- as.numeric( trees[[i]]$node.label[diffs_nodes - numtips] )
    diffs_BL_mean   <- mean( diffs_BL, na.rm=TRUE )
    diffs_BS_mean   <- mean( diffs_BS, na.rm=TRUE )
    
    sames_nodes     <- which(diffs==1)+length(trees[[i]]$tip.label)
    sames_edges     <- which(trees[[i]]$edge[,2] %in% sames_nodes)
    sames_BL        <- trees[[i]]$edge.length[sames_edges]
    sames_BS        <- as.numeric( trees[[i]]$node.label[sames_nodes - numtips] )
    sames_BL_mean   <- mean( sames_BL, na.rm=TRUE )
    sames_BS_mean   <- mean( sames_BS, na.rm=TRUE )
    
    params_subsets<- data_info( d, i )  # get num of params and subsets for this dataset-pfscheme combo
    
    slope = node.depth.slope(trees.rooted[[i]])# get the slope of the regression of Branch Length on Branch depth.
                                        # This is to test whether under-partitioned models tend to systematically underestimate the length of deeper branches in the tree

    row <- data.frame( dataset=d, PF.scheme=pfschemes[i], dist.MS=MS, dist.RF=RF, dist.PD=PD, dist.KF=KF,
                       nodes.total=totnodes, nodes.internal=nnodes, nodes.numdiff=length(which(diffs==0)), Diffs.nodes=paste(diffs_nodes, collapse=','), 
                       Diffs.BS=paste(diffs_BS, collapse=','), Diffs.BS.mean=diffs_BS_mean, Diffs.BL=paste(diffs_BL, collapse=','), Diffs.BL.mean=diffs_BL_mean,
                       Sames.BS=paste(sames_BS[-1], collapse=','), Sames.BS.mean=sames_BS_mean, Sames.BL=paste(sames_BL, collapse=','), Sames.BL.mean=sames_BL_mean,
                       BL.mean=BL_mean, BS.mean=BS_mean, BL.all = paste(BL_all, collapse=','), BS.all = paste(BS_all, collapse=','),  params=params_subsets[1], subsets=params_subsets[2], 
                       slope.BLdepth = slope,
                       row.names=NULL, stringsAsFactors=FALSE )
    
    outdf <- rbind(outdf, row)
    
  }
  return(outdf)
  
  #   combinedData  <- data.frame() # nice happy data frame containing all nodes and edges of all 4 trees
  #   for(i in 1:length(trees))
  #   {
  #     combinedData  <- rbind( combinedData, toDataFrame(trees.phylo4[[i]]) ) 
  #   }
}


#############   init vars   #####################

basepath      <- "C:/devwork/partitionfinder/Project_PFanalysis/"
treeCmppath   <- "C:/Downloaded/Phylogenetics/TreeCmp/bin/TreeCmp.jar"

# expected tree filename format: "RAxML_bipartitions.<partitionscheme>.final"
 datasets      <- list("Anderson_2013", "Bergsten_2013", "Broughton_2013", "Brown_2012", "Caterino_2001", "Cognato_2001", 
                      "Day_2013", "Devitt_2013", "Dornburg_2012", "Dsouli_2011", "Ekrem_2010", "Elias_2009", 
                      "Fishbein_2001", "Fong_2012", "Grande_2013", "Guschanski_2013", "Kang_2013a", "Kang_2013b", "Kaffenberger_2011", 
                      "Kawahara_2013", "Lartillot_2012", "Leavitt_2013", "Li_2008", "Murray_2013", "Rightmyer_2013", 
                      "Sauquet_2011", "Seago_2011", "Sharanowski_2011", "Siler_2013", "Tolley_2013", "Unmack_2013", "Wainwright_2012", 
                      "Ward_2010", "Welton_2013"
                      )     # list your dataset folder names here!

# tracks which model was best for each pfscheme within each dataset (1=GTRIG, 0=GTRG). I Manually entered this data from PF outputs.
bestmodels    <- list(Anderson_2013=c(1,1,1,1), Bergsten_2013=c(1,1,1,1), Broughton_2013=c(1,1,1,1), Brown_2012=c(1,1,1,1), Caterino_2001=c(1,1,1,1), Cognato_2001=c(1,1,1,1), 
                      Day_2013=c(1,1,1,1), Devitt_2013=c(1,1,0,1), Dornburg_2012=c(1,1,1,1), Dsouli_2011=c(0,1,0,1), Ekrem_2010=c(1,1,1,1), Elias_2009=c(1,1,1,1), 
                      Fishbein_2001=c(1,1,1,1), Fong_2012=c(1,1,0,1), Grande_2013=c(1,1,1,1), Guschanski_2013=c(1,1,1,1), Kang_2013a=c(1,1,0,1), Kang_2013b=c(1,1,1,1), Kaffenberger_2011=c(1,1,1,1), 
                      Kawahara_2013=c(1,1,1,1), Lartillot_2012=c(1,1,1,1), Leavitt_2013=c(1,1,1,1), Li_2008=c(1,1,1,1), Murray_2013=c(1,1,1,1), Rightmyer_2013=c(1,1,1,1), 
                      Sauquet_2011=c(1,1,1,1), Seago_2011=c(1,1,1,1), Sharanowski_2011=c(1,1,1,1), Siler_2013=c(1,1,1,1), Tolley_2013=c(1,1,1,1), Unmack_2013=c(1,1,1,1), Wainwright_2012=c(1,1,1,1), 
                      Ward_2010=c(1,1,1,1), Welton_2013=c(1,1,1,1))

pfschemes     <- c("PFbic","PFaicc", "PFall","PFnone")            # Partitioning scheme names. These must be in your tree filenames.
fsuffix       <- "final"                                       # this is the end part of the tree filename.



######################################################################################################   
#####  read in the trees for each dataset and create core dataframe                 ##################
######################################################################################################  

getCoredata <- function(vs_tree='PFbic') {
  # Reads in the trees for each dataset, pulls them apart and creates core dataframe with data on conflicts, topology BS and BL values.
  #
  # Args:
  #   vs_tree: char string. One of the partitioning schemes to be used as a basis for comparison. Either PFbic, PFaicc, PFall, PFnone
  #
  # Returns: data frame with all the yummy tree stats. Also saves out an RData file for use elsewhere.
  
  coredata <- do.call(rbind,lapply(datasets, doOneDataset, vs_tree))
  coredata$PF.scheme <- as.factor(coredata$PF.scheme)
  coredata$dataset <- as.factor(coredata$dataset)
  save.image(file='coredata.RData')
  return(coredata)
}
